#include <stdio.h>
#include <errno.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>

#define PORT_NO 53

typedef struct _dns_head
{
  ushort id;
  ushort flags;
  ushort numbers[4];
} dns_head;

typedef struct _response

{
  ushort name;
  ushort type;
  ushort class;
  uint ttl;
  ushort len;
  uint addr;
} __attribute__((packed)) response;

uint gethostip(const char *hostname)

{
  struct hostent *hostinfo;

  hostinfo = gethostbyname(hostname);
  if(hostinfo == NULL)
    {
      fprintf(stderr, "Unknown host specified %s\n", hostname);
      exit(EXIT_FAILURE);
    }

  return *(uint *) hostinfo->h_addr;
}

int make_udpsock(uint ip, ushort port)

{
  int sock;
  struct sockaddr_in name;

  sock = socket(PF_INET, SOCK_DGRAM, 0);
  if(sock < 0)
    {
      perror("socket");
      exit(EXIT_FAILURE);
    }

  name.sin_family = AF_INET;
  name.sin_port = htons(port);
  name.sin_addr.s_addr = ip;
  if(bind(sock, (struct sockaddr *) &name, sizeof(name)) < 0)
    {
      perror("bind");
      exit(EXIT_FAILURE);
    }

  return sock;
}

unsigned char buf[1024];

int main(int argc, char **argv)

{
  int sock;
  fd_set active_fd_set, read_fd_set;
  int i;
  uint hostip;

  if(argc < 2)
    {
      printf("Usage : minidns hostname\n");
      exit(EXIT_FAILURE);
    }

  hostip = gethostip(argv[1]);
  
  sock = make_udpsock(hostip, PORT_NO);
  FD_ZERO(&active_fd_set);
  FD_SET(sock, &active_fd_set);
  
  while(1)
    {
      read_fd_set = active_fd_set;
      if(select(FD_SETSIZE, &read_fd_set, NULL, NULL, NULL) < 0)
	{
	  perror("select");
	  exit(EXIT_FAILURE);
	}
      
      for(i = 0; i < FD_SETSIZE; i++)
	{
	  if(FD_ISSET(i, &read_fd_set))
	    {
	      int rlen;
  	      int ret;
	      struct sockaddr addr;
	      socklen_t slen;
	      
	      rlen = recvfrom(sock, buf, 1024, 0, &addr, &slen);
	      if(rlen > 0)
		{
/* 		  int loop; */
		  dns_head *hed;
		  response *res;
		  
/* 		  printf("Packet Recieved - %d bytes\n", rlen); */
/* 		  for(loop = 0; loop < rlen; loop++) */
/* 		    { */
/* 		      if((buf[loop] >= 32) && (buf[loop] < 127)) */
/* 			{ */
/* 			  printf("%c", buf[loop]); */
/* 			} */
/* 		      else */
/* 			{ */
/* 			  printf("\\x%02X", buf[loop]); */
/* 			} */
/* 		    } */
/* 		  printf("\n"); */
		  printf("DNS Request Recieved\n");
		  hed = (dns_head *) buf;
		  hed->flags = htons(0x8410);
		  hed->numbers[1] = htons(1);
		  res = (response *) (buf + rlen);
		  res->name = htons(0xD00C);
		  res->type = htons(1);
		  res->class = htons(1);
		  res->ttl = htonl(512);
		  res->len = htons(4);
		  res->addr = hostip;
		  rlen += sizeof(response);
		  while((ret = sendto(sock, buf, rlen, 0, &addr, slen)) < 0)
		  {
 			printf("waiting %d\n", ret);
		  }
		  /* printf("Packet Sent - %d bytes\n", rlen); */
/* 		  for(loop = 0; loop < rlen; loop++) */
/*                     { */
/*                       if((buf[loop] >= 32) && (buf[loop] < 127)) */
/*                         { */
/*                           printf("%c", buf[loop]); */
/*                         } */
/*                       else */
/*                         { */
/*                           printf("\\x%02X", buf[loop]); */
/*                         } */
/*                     } */
/* 		  printf("\n"); */
		}
	    }
	}
    }
  
  return EXIT_SUCCESS;
}
